(function($) {
	window.App = {
		Collections: {
			CategoryList: Backbone.Collection.extend({
				model: Backbone.Model.extend()
			}),
			ArticleList: Backbone.Collection.extend({
				model: Backbone.Model.extend(),
				shownArticles: [],
				getThis: function() {
					return $.extend(true, {}, this);
				},

				/**
				 * finds articles based on a specified array of article IDs
				 * @param {number Array} articleIDs
				 * @param {boolean} skipShown
				 * @param {boolean} addToSkipped
				 * @return {ArticleList Collection} results
				 */
				findArticles: function(articleIDs, skipShown, addToSkipped) {
					var results = [],
						model,
						that = window.App,
						collection = this.getThis(),
						models = skipShown ? this.getUnshownCollection().models : collection.models;


					for (i = 0; i < models.length; i++) {
						model = models[i].attributes;

						for (j = 0; j < articleIDs.length; j++) {
							if (model.ID === articleIDs[j]) {
								//strip tags
								model.Title = that._stripTags(model.Title); 
								results.push(model);

								if (articleIDs.length == 1) {
									collection.previousArticle = i > 0 ? models[i - 1].attributes.ID : null;
									collection.nextArticle = i < models.length - 1 ? models[i + 1].attributes.ID : null;
								}

								//remove found ID from article ID list
								articleIDs.splice(j, 1);

								if (addToSkipped) {
									//add model ID to shown list 
									this.shownArticles.push(model.ID);
								}
							}
						}
					}
					return collection.reset().add(results);
				}
			})
		},
		data: {},
		imgVars : {
			'pad' : 30,
			'raw_width' : pageWidth,
			'raw_height' : pageHeight,
			'max_width' : 640
		},
		articleList: {},
		categoryList: {},
		status: {
			activePage: 'home',
			categoryColorMap: {},
			currentCategory: '',
			currentArticle: {},
			previousArticle: '',
			nextArticle: '',
			fontSizeLevel: 0
		},
		sectionTemplateIDs: {			
			full: {
				"section": "fullArticle-content",
				"template": "fullViewTpl"
			}
		},
		templates: {},
		themes: {},
		categoryColors: ['red', 'green', 'blue','mint', 'violet', 'aqua',  'pink', 'orange'],
		_init: function() {			
			var that = this,
				body = $('body');
				
				that.data = issuecontent.issue_content;
				var options = eval("(" + issuecontent.options + ")");

			that._reMapData();

			that.articleList = new this.Collections.ArticleList(that.data.IssueContents);	

			that._showArticle(articleID);		

			//update the footer
			$('.copyYear').html(new Date().getFullYear());
			
			//activate buttons
			body.on('click', '.icon-plustext', function() {				
				that._resizeText(1, 'resizable-text');
			});
			body.on('click', '.icon-minustext', function() {
				that._resizeText(-1, 'resizable-text');
			});
			body.on('click', '.social-facebook', function(e) {
                e.preventDefault();
                var a = that.status.currentArticle;
                window.location.href="https://www.facebook.com/sharer/sharer.php?u=" + pubUrl + "%2F%3Fiid%3d" + iid + "%26startpage%3D"+ startpage + "%26ArticleTitle%3D" + a.ID;
            });
            body.on('click', '.social-twitter', function(e) {
                e.preventDefault();
                var a = that.status.currentArticle,
                    status = encodeURI(that.status.currentArticle.Title + '\n' + 'More at ');
                window.location.href="https://twitter.com/intent/tweet?text=" + status + pubUrl + "%2F%3Fiid%3d" + iid + "%26startpage%3D"+ startpage + "%26ArticleTitle%3D" + a.ID + "%23";
            });
            body.on('click', '.social-email', function(e) {
                e.preventDefault();
                var a = that.status.currentArticle;
                var msg = encodeURIComponent(a.Title + '\n') + encodeURIComponent(a.Abstract + '\n\nRead more at:\n')  + encodeURIComponent(pubUrl)  + encodeURIComponent('/?iid=') + iid + encodeURIComponent('&startpage=')+ startpage + encodeURIComponent('&ArticleTitle=') + a.ID;
                var articleShareLongURL = encodeURIComponent(pubUrl)  + encodeURIComponent('/?iid=') + iid + encodeURIComponent('&startpage=')+ startpage + encodeURIComponent('&ArticleTitle=') + a.ID;
                var articleShareShortURL;
                $.post("https://www.googleapis.com/urlshortener/v1/url", {"longUrl":articleShareLongURL}, function(result){articleShareShortURL = result.id});
                var subject = encodeURI(a.Title);
                var emailLink = "#CORONAmailto:?subject=" + subject + "&body=" + msg;
                window.location.href = "#CORONAmailto:?subject=" + subject + "&body=" + msg;
            });
						

			//add event listeners
			// Listen for orientation changes
			window.addEventListener("orientationchange", function() {
				that._resizeImage();
			}, false);
			window.addEventListener("resize", function(){				
				that._resizeImage();
			});

			//set Font size based on cookie			
			// that._setCookie('fontSizeLevel',0)
			var fontSizeLevel = parseInt( that._getCookie('fontSizeLevel') );
			if ( fontSizeLevel == "" || fontSizeLevel<0 ){
				fontSizeLevel = 0;
			}
			that.status.fontSizeLevel = fontSizeLevel;
			var fontSize = 1 + (fontSizeLevel * .2);			
			$('.resizable-text').css('fontSize', fontSize + 'em');

		},
		_showArticle : function(id){
				var that = window.App,
					sid = that.sectionTemplateIDs,
					a;
					
				var al = that.articleList.findArticles([parseInt(id)]); 


				
				if (al.models.length > 0) {
					a =  al.models[0].attributes; 
					var c = $('#' + sid.full.section).html('');								

					if (!a.SubHeading || a.SubHeading == ''){
						$('.subheading').hide();
					}

					//set webpage title
					$('title').html( a.Title );		

					//set fontsize
					fontSizeLevel = that.status.fontSizeLevel
					var fontSize = 1 + (fontSizeLevel * .2);			
					$('.resizable-text').css('fontSize', fontSize + 'em');	
					
					that.status.currentArticle = al.models[0].attributes;

					//render template view
					var FullArticle = that._makeSectionView(sid.full.section, sid.full.template, al);				
					new FullArticle();

					//add article menus
					var articleMenu = _.template( $( "#articleMenu" ).html()  );
					$('.preview-full .img-info').after(articleMenu);
					$('.preview-full').append(articleMenu);
				} 
		},
		_resizeText: function(multiplier, selectorClass) {
			var max = 2.5,
				min = 1,
				level = this.status.fontSizeLevel;
				text = document.getElementsByClassName(selectorClass);

			for (i = 0; i < text.length; i++) {
				t = text[i];

				if (t.style.fontSize == "") {
					t.style.fontSize = "1.0em";
				}

				var newSize = parseFloat(t.style.fontSize) + (multiplier * 0.2);

				if (newSize <= min) {
					newSize = 1;
					level = 0;
				}
				if (newSize < max) {
					t.style.fontSize = newSize + "em";
					level = multiplier>0? level+1 : level-1;
				}
			}			
			this.status.fontSizeLevel = level;
			//set cookie
			this._deleteCookie('fontSizeLevel');
			this._setCookie('fontSizeLevel', level);
		},
		_makeSectionView: function(sectionID, templateID, collection) {
			return Backbone.View.extend({
				el: $("#" + sectionID),

				initialize: function() {
					if (collection) {
						this.collection = collection;
						this.render();
					}
				},

				render: function() {
					var that = this;
					_.each(this.collection.models, function(item) {
						that.renderSection(item);
					}, this);
				},

				renderSection: function(item) {
					var template = _.template($("#" + templateID).html());
					var article = template(item.attributes);
					this.$el.append(article);
				}
			});
		},
		_reMapData: function() {
			var that = this;
			
			if (that.data) {
				var colors = this.categoryColors,
					colorMap = {},
					articles = that.data.IssueContents,
					catArray = new that.Collections.CategoryList(),
					colorIndex = 0,
					article,
					category,
					
					a,
					width = 370;
					
				for (i = 0; i < articles.length; i++) {
					a = articles[i];
					
					colorIndex = this._makeCategoryColor(a, colorMap, catArray, colorIndex);
					this.status.categoryColorMap = colorMap;
					this.categoryList = catArray;
					var catName = a.IssueContentType.Name;
					catName = catName.replace(/\W+/g, '').toLowerCase();					
					a.categoryClass = catName;
					
					var temp = a.PageName.split('page');
					var pageNum = parseInt(temp[1]);										

					this._makeImage(a);
				}
			}
		},
		_makeCategoryColor: function(a, colorMap, catArray, colorIndex) {
			var colors = this.categoryColors,
				category = a.IssueContentType.Name;
			a.category = category;

			if (colorMap.hasOwnProperty(category)) {
				//set category color  
				a.categoryColor = colorMap[category];
			} else {
				a.IssueContentType.Color = colors[colorIndex];
				catArray.add(a.IssueContentType);
				//add new category to color map and assign a color
				colorMap[category] = colors[colorIndex];
				a.categoryColor = colors[colorIndex];
				colorIndex++;

				if (colorIndex == colors.length) {
					colorIndex = 0;
				}
			}
			return colorIndex;

		},
		_makeImage: function(a) {
			var img, src, that = this;
			
			if (a.Images.length > 0 && a.Images != "") {

				a.Images = eval(a.Images);				

				img = a.Images;

				img.src = a.Images != "" ?   a.PageName + '.jpg' : img.src = "";
				
				if (a.Images != ""){
					var coords = eval(img)[0].coordinates,	
						i = that._translateCoords( coords );																						
					img.zoom = i.zoom
					img.clipHeight = i.clipHeight;
					img.top =  i.top;
					img.left= i.left;
					img.coords = coords;
				}				

				if ( img[0] && (img[0].caption && img[0].caption != "") ) {
					img.caption = decodeURIComponent(img[0].caption);
					img.captionClass = "";
				} else {
					img.caption = "";
					img.captionClass = "noCaption";
				}

			}

		},
		_resizeImage : function(){		

			var fig = $('.clipzone').first(),
				img = fig.find('img').first(),
				coords = $('.clipzone').first().attr('coords');
			if ( ! img.hasClass('noimage') ){
				i = this._translateCoords( coords );
				
				fig.css({
					'height' : i.clipHeight
				});	
				img.css({
					'zoom' : i.zoom,
					'top' :  i.top,
					'left' : i.left
				});
			}	

		},
		_translateCoords : function(coords){
			//arrCoords [ 0=>left, 1=>top, 2=>right, 3=>bottom ]	
			var i = {},
				arrCoords = coords.split(","),
				iv = this.imgVars,
				pad = iv.pad,
				full_width = iv.raw_width,
				full_height = iv.raw_height,

				img_width = (arrCoords[2] -  arrCoords[0])/100 * full_width,
				img_height = (arrCoords[3] -  arrCoords[1])/100 * full_height,
				maxWidth = window.innerWidth > iv.max_width? iv.max_width : window.innerWidth;

			var zoom = ( maxWidth-pad ) / img_width,

				img_top = arrCoords[1]/100 * full_height,
				img_left = arrCoords[0]/100 * full_width;
														
			i.zoom = (zoom * 100) + '%';
			i.clipHeight = (img_height * zoom) + 'px';
			i.top =  '-' + img_top + 'px';
			i.left= '-' + img_left + 'px';					

			return i;
		},
		_stripTags: function(s){
			$('body').append('<div id="toTextTemp"></div>');
			var temp = $('#toTextTemp');
			temp.hide().html(s);
			var text = temp.text().replace(/\r?\n|\r/g, ' ');
			temp.remove();

			return text;
		},
		_setCookie: function(cookie_name, cookie_value){
			var lifespan_in_days = 1;
			document.cookie = cookie_name + "=" + encodeURIComponent( cookie_value ) +
     			 "; max-age=" + 60 * 60 * 24 * lifespan_in_days;
		},
		_getCookie: function(cookie_name){
			var cookie_string = document.cookie ;
			if (cookie_string.length != 0) {
				var cookie_value = cookie_string.match ( '(^|;)[\s]*' + cookie_name + '=([^;]*)' );
				if (cookie_value){
					return decodeURIComponent ( cookie_value[2] ) ;
				}				
			}
			return 'none' ;
		},
		_deleteCookie: function(cookie_name){
		  document.cookie = cookie_name + "=; max-age=0;" ;
		},
		_imgError: function(image) {
			image.onerror = "";
			//image.src ="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABAQMAAAAl21bKAAAAA1BMVEUAAACnej3aAAAAAXRSTlMAQObYZgAAAApJREFUCNdjYAAAAAIAAeIhvDMAAAAASUVORK5CYII%3D";			
				image.src = "../html/noimage.png";
        image.alt="";
			$(image).addClass("noimage").removeAttr('style');
			return true;
		}
	}
	/* to prevent conflict with .net */
	// _.templateSettings = {
	//   interpolate: /\{\{(.+?)\}\}/g
	// };
}(jQuery));